﻿using HotelContracts.BindingModels;
using HotelContracts.SearchModels;
using HotelContracts.StoragesContracts;
using HotelContracts.ViewModels;
using HotelDataBaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HotelDataBaseImplement.Implemets
{
    public class ConferenceBookingStorage : IConferenceBookingStorage
    {
        public ConferenceBookingViewModel? Delete(ConferenceBookingBindingModel model)
        {
            using var context = new HotelDataBase();

            var element = context.ConferenceBookings
                .FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                context.ConferenceBookings.Remove(element);
                context.SaveChanges();

                return element.GetViewModel;
            }

            return null;
        }

        public ConferenceBookingViewModel? GetElement(ConferenceBookingSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new HotelDataBase();

            return context.ConferenceBookings
                .Include(x => x.Dinners)
                .ThenInclude(x => x.Dinner)
                .ThenInclude(x => x.RoomDinners)
                .ThenInclude(x => x.Room)
                .Include(x => x.Conference)
                .Include(x => x.Headwaiter)
                .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?
                .GetViewModel;
        }

        public List<ConferenceBookingViewModel> GetFilteredList(ConferenceBookingSearchModel model)
        {
            if (!model.DateFrom.HasValue && !model.DateTo.HasValue && !model.HeadwaiterId.HasValue)
            {
                return new();
            }
            using var context = new HotelDataBase();

            if (model.DateFrom.HasValue)
            {
                return context.ConferenceBookings
                        .Include(x => x.Dinners)
                        .ThenInclude(x => x.Dinner)
                        .ThenInclude(x => x.RoomDinners)
                        .ThenInclude(x => x.Room)
                        .Include(x => x.Conference)
                        .Include(x => x.Headwaiter)
                        .Where(x => x.Conference.StartDate >= model.DateFrom && x.Conference.StartDate <= model.DateTo)
                        .Select(x => x.GetViewModel)
                        .ToList();
            }
            else if (model.HeadwaiterId.HasValue)
                return context.ConferenceBookings
                    .Include(x => x.Dinners)
                    .ThenInclude(x => x.Dinner)
                    .ThenInclude(x => x.RoomDinners)
                    .ThenInclude(x => x.Room)
                    .Include(x => x.Conference)
                    .Include(x => x.Headwaiter)
                    .Where(x => x.HeadwaiterId == model.HeadwaiterId)
                    .Select(x => x.GetViewModel)
                    .ToList();

            return context.ConferenceBookings
                .Include(x => x.Dinners)
                .ThenInclude(x => x.Dinner)
                .ThenInclude(x => x.RoomDinners)
                .ThenInclude(x => x.Room)
                .Include(x => x.Conference)
                .Include(x => x.Headwaiter)
                .Where(x => x.Id == model.Id)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<ConferenceBookingViewModel> GetFullList()
        {
            using var context = new HotelDataBase();

            return context.ConferenceBookings
                .Include(x => x.Dinners)
                .ThenInclude(x => x.Dinner)
                .ThenInclude(x => x.RoomDinners)
                .ThenInclude(x => x.Room)
                .Include(x => x.Conference)
                .Include(x => x.Headwaiter)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public ConferenceBookingViewModel? Insert(ConferenceBookingBindingModel model)
        {
            using var context = new HotelDataBase();
            var newConferenceBooking = ConferenceBooking.Create(context, model);

            if (newConferenceBooking == null)
            {
                return null;
            }

            context.ConferenceBookings.Add(newConferenceBooking);
            context.SaveChanges();

            return context.ConferenceBookings
                .Include(x => x.Dinners)
                .ThenInclude(x => x.Dinner)
                .ThenInclude(x => x.RoomDinners)
                .ThenInclude(x => x.Room)
                .Include(x => x.Conference)
                .Include(x => x.Headwaiter)
                .FirstOrDefault(x => x.Id == newConferenceBooking.Id)
                ?.GetViewModel;
        }


        public ConferenceBookingViewModel? Update(ConferenceBookingBindingModel model)
        {
            using var context = new HotelDataBase();

            var conferenceBooking = context.ConferenceBookings
                .Include(x => x.Dinners)
                .ThenInclude(x => x.Dinner)
                .ThenInclude(x => x.RoomDinners)
                .ThenInclude(x => x.Room)
                .Include(x => x.Conference)
                .Include(x => x.Headwaiter)
                .FirstOrDefault(x => x.Id == model.Id);

            if (conferenceBooking == null)
            {
                return null;
            }

            conferenceBooking.Update(model);
            context.SaveChanges();

            return conferenceBooking.GetViewModel;
        }
    }
}
